<?php

namespace App\Console\Commands;

use App\Console\CrawlerHelper;
use Exception;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;

/**
 * Database migration
 *
 * Refresh Materialized View (Report DGP) data
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */
class RefreshReportDGP extends Command
{
    /**
     * The console command description.
     */
    protected $description = 'Refresh data on pgsql\'s Materialized View (reportDGP)';

    /**
     * The name and signature of the console command.
     */
    protected $signature = 'refresh:report';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $return = 0;

        try {
            DB::statement('REFRESH MATERIALIZED VIEW reportDgp;');
            $this->line('Report DGP: Materialized View updated successful.');

            app(CrawlerHelper::class)->sendNotification('Refreshed Report MV');
        } catch (Exception $e) {
            $this->error('Report DGP. Failed to refresh data. Error message: ' . $e->getMessage());

            app(CrawlerHelper::class)->sendNotification('Failed to refresh MV. Error message: ' . $e->getMessage());

            $return = 1;
        }

        return $return;
    }
}
