<?php

namespace App\Console\Commands;

use Exception;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;

/**
 * Database migration
 *
 * Refresh Materialized View (Report DGP) data
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */
class RefreshReportDgpDraft extends Command
{
    /**
     * The console command description.
     */
    protected $description = 'Refresh data on pgsql\'s Materialized View (reportdgp_draft)';

    /**
     * The name and signature of the console command.
     */
    protected $signature = 'refresh:report_draft';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $return = 0;

        try {
            DB::statement('REFRESH MATERIALIZED VIEW reportdgp_draft;');
            $this->line('Report DGP Draft: Materialized View updated successful.');
        } catch (Exception $e) {
            $this->error('Report DGP Draft. Failed to refresh data. Error message: ' . $e->getMessage());

            $return = 1;
        }

        return $return;
    }
}
