/* eslint-disable promise/param-names */
/* eslint-disable no-void */
/* eslint-disable new-cap */
/* eslint-disable object-curly-spacing */
/* eslint-disable indent */
/* eslint-disable max-len */

const chromium = require('@sparticuz/chromium');
require('dotenv').config();

const { addExtra } = require('puppeteer-extra');
const puppeteer = require('puppeteer-core');
const puppeteerExtra = addExtra(puppeteer);

const stealthPlugin = require('puppeteer-extra-plugin-stealth');
puppeteerExtra.use(stealthPlugin());

const [, , ...args] = process.argv;

/**
 * Run test
 *
 * @param {string} link
 * @param {string} additional
 * @param {string} requests
 * @param {string} textOnly
 */
async function test(link, additional = '{}', requests, textOnly) {
  // console.log(await chromeLambda.executablePath);
  // return;
  let ageGateBypass = null;
  try {
    ageGateBypass = JSON.parse(additional).agegate;
  } catch {
    ageGateBypass = null;
  }

  let agegateStorage = '';
  let agegateData = '';

  if (ageGateBypass) {
    agegateStorage = ageGateBypass.storage;
    agegateData = ageGateBypass.data;
  }

  const final = {};

  const browser = await puppeteerExtra.launch({
    headless: true,
    ignoreHTTPSErrors: true,
    args: [
      '--no-sandbox',
      '--disable-setuid-sandbox',
      '--window-size=1920,1080',
      '--lang=pt-BR,pt',
      '--disable-sync',
      '--disable-translate',
      '--no-first-run',
      '--safebrowsing-disable-auto-update',
      '--disable-prompt-on-repost',
      '--disable-popup-blocking',
      '--disable-default-apps',
      '--password-store=basic',
      '--use-mock-keychain',
      '--disable-crash-reporter',
      "--proxy-server='direct://'",
      '--proxy-bypass-list=*',
      '--deterministic-fetch',
    ],
    executablePath: await chromium.executablePath,
    defaultViewport: chromium.defaultViewport,
  });

  const page = await browser.newPage();

  // Increase the default timeout limit (2 minutes)
  await page.setDefaultNavigationTimeout(120000);

  // We need to intercept the requests to store
  const requestList = {
    allowAll: [],
    initial: [],
  };

  let allowedAllCookies = false;

  if (requests === true) {
    await page.setRequestInterception(true);

    page.on('request', (request) => {
      if (allowedAllCookies) {
        requestList.allowAll.push(request.url());
      } else {
        requestList.initial.push(request.url());
      }

      request.continue();
    });
  }

  let sourceCode = '';

  try {
    let response = [];
    // We'll use promise race to ensure page dont exced our timeout (1.5 minute)

    response = await Promise.race([
      page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
      new Promise((x) => setTimeout(x, 115 * 1000)),
    ]);

    await page.waitForTimeout(10000);

    if (response === undefined || response.status() >= 400) {
      // Let's try to open with a https - some links isnt redirecting properly
      link = link.replace('http://', 'https://');

      response = await Promise.race([
        page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
        new Promise((x) => setTimeout(x, 115 * 1000)),
      ]);

      await page.waitForTimeout(10000);

      if (response === undefined) {
        throw new Error('Unexpected problem with the link');
      }

      // Due to the change from waiting networkIdle to DOMContentLoadade, additional timeout
      await page.waitForTimeout(15000);
    }

    if (response && response.status() >= 400) {
      final.onetrustImplemented = false;
      final.siteStatus = 'offline';

      throw new Error('Invalid HTTP Status');
    }

    sourceCode = await page.content();

    const linkFinal = page.url();

    const finalUrl = new URL(linkFinal);

    const initialUrl = new URL(link);

    const social =
      initialUrl.hostname.includes('facebook.com') ||
      initialUrl.hostname.includes('instagram.com') ||
      initialUrl.hostname.includes('youtube.com');

    if (
      !social &&
      finalUrl.hostname.replace('www.', '').toLowerCase() !==
        initialUrl.hostname.replace('www.', '').toLowerCase()
    ) {
      throw new Error('Redirect deteted');
    }

    // Used to ensure, we'll not get same iframe twice
    const iframeList = [];

    for (const frame of page.frames()) {
      const iframeUrl = await frame.url();

      if (
        iframeUrl &&
        iframeUrl.length > 3 &&
        iframeUrl.indexOf('.') !== -1 &&
        !iframeList.includes(iframeUrl)
      ) {
        const domainIframe = new URL(iframeUrl);

        if (
          domainIframe &&
          domainIframe.hostname.replace('www.', '') !==
            initialUrl.hostname.replace('www.', '')
        ) {
          sourceCode += '\n\n' + (await frame.content());
          iframeList.push(iframeUrl);
        }
      }
    }

    if (agegateData.length > 0) {
      switch (agegateStorage) {
        case 'cookie':
        case 'cookies':
          await page.evaluate((agegateData) => {
            const d = new Date();
            d.setTime(d.getTime() + 30 * 24 * 60 * 60 * 1000);
            const expires = 'expires=' + d.toUTCString();

            agegateData.forEach((ageData) => {
              document.cookie =
                ageData.key + '=' + ageData.value + ';' + expires + ';path=/';
            });
          }, agegateData);

          break;

        case 'session_storage':
          await page.evaluate((agegateData) => {
            agegateData.forEach((ageData) => {
              sessionStorage.setItem(ageData.key, ageData.value);
            });
          }, agegateData);
          break;

        case 'local_storage':
          await page.evaluate((agegateData) => {
            agegateData.forEach((ageData) => {
              localStorage.setItem(ageData.key, ageData.value);
            });
          }, agegateData);
          break;

        case 'redirect':
          await Promise.race([
            page
              .goto(agegateData[0].value, { waitUntil: 'domcontentloaded' })
              .catch((e) => void e),
            new Promise((x) => setTimeout(x, 60 * 1000)),
          ]);
          break;

        case 'steps':
          for await (const step of agegateData) {
            switch (step.action) {
              case 'fill':
                await page.waitForSelector(step.selector, {
                  visible: true,
                  timeout: 3000,
                });
                await page.focus(step.selector);
                await page.keyboard.type(step.value, { delay: 200 });
                break;
              case 'click':
                await page.waitForSelector(step.selector);
                await page.click(step.selector);
                break;
              case 'select':
                await page.waitForSelector(step.selector, {
                  visible: true,
                  timeout: 3000,
                });
                await page.select(step.selector, step.value);
                await page.waitForTimeout(1000);
                break;
              case 'end':
                await page.waitForTimeout(5000);
                break;
              default:
                break;
            }
          }
          break;
        default:
          break;
      }
    }

    await Promise.race([
      page.goto('about:blank').catch((e) => void e),
      new Promise((x) => setTimeout(x, 5 * 1000)),
    ]);

    // We only need to go again to the website, when:
    // Bypass age-gate or need network requests
    if (agegateData.length > 0 || requests === true) {
      // We'll use promise race to ensure page dont exced our timeout (1.5 minute)
      await Promise.race([
        page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
        new Promise((x) => setTimeout(x, 115 * 1000)),
      ]);

      await page.waitForTimeout(10000);
    }

    if (agegateData.length > 0) {
      sourceCode += '\n\n' + (await page.content());

      // Use this to get the content from iframes inside the page
      for (const frame of page.frames()) {
        const iframeUrl = await frame.url();

        if (
          iframeUrl &&
          iframeUrl.length > 3 &&
          iframeUrl.indexOf('.') !== -1 &&
          !iframeList.includes(iframeUrl)
        ) {
          const domainIframe = new URL(iframeUrl);

          if (
            domainIframe &&
            domainIframe.hostname.replace('www.', '') !==
              initialUrl.hostname.replace('www.', '')
          ) {
            sourceCode += '\n\n' + (await frame.content());
            iframeList.push(iframeUrl);
          }
        }
      }
    }

    if (requests === true) {
      await page.waitForTimeout(5000);

      allowedAllCookies = true;

      await page.evaluate(() => {
        if (typeof Optanon !== 'undefined') {
          try {
            // eslint-disable-next-line no-undef
            Optanon.AllowAll();
          } catch (error) {}
        }
      });

      await page.waitForTimeout(10000);

      final.requests = requestList;
    }

    final.sourceCode = sourceCode;

    let innerText = '';
    const arrData = [];

    // If we need the text only version of the rendered page
    if (textOnly === true) {
      innerText = await page.evaluate(() => {
        return document.querySelector('body').innerText;
      });
      arrData.additional_data = innerText;
    }

    final.status = 'success';
  } catch (error) {
    final.status = 'failed';
    final.error = error.message;
    final.sourceCode = '';
  } finally {
    const finalCoded = JSON.stringify(final);
    console.log(finalCoded);

    browser.close();

    if (final.status === 'success') {
      process.exit();
    } else {
      process.exit(1);
    }
  }
}

const link = args[0];
const additional = args[1];
const requests = args[2] || true;
const textOnly = args[3] || false;

test(link, additional, requests, textOnly);
