/* eslint-disable no-undef */
/* eslint-disable promise/param-names */
/* eslint-disable no-void */
const chromium = require('@sparticuz/chromium');

const { addExtra } = require('puppeteer-extra');
const puppeteer = require('puppeteer-core');
const puppeteerExtra = addExtra(puppeteer);

// add stealth plugin and use defaults (all evasion techniques)
const stealthPlugin = require('puppeteer-extra-plugin-stealth');
puppeteerExtra.use(stealthPlugin());

exports.run = async function (req, res) {
  let link = req.query.link || (req.body && req.body.link);

  // TODO: send this informatoin to GCP CF
  const additional = '{}';

  let responseCode = 200;

  let ageGateBypass = null;

  try {
    ageGateBypass = JSON.parse(additional).agegate;
  } catch {
    ageGateBypass = null;
  }

  let agegateStorage = '';
  let agegateData = '';

  if (ageGateBypass) {
    agegateStorage = ageGateBypass.storage;
    agegateData = ageGateBypass.data;
  }

  let final = {};

  const agegateArray = [];

  if (agegateData && agegateData.length > 0) {
    agegateData.forEach((entry) => {
      agegateArray.push(entry.key);
    });
  }

  final.cookieBannerImplemented = true;
  final.cookieBannerDomain = '';
  final.cookieBannerSameDomain = false;
  final.siteStatus = 'live';

  const browser = await puppeteerExtra.launch({
    headless: true,
    args: [
      '--no-sandbox',
      '--disable-setuid-sandbox',
      '--window-size=1920,1080',
      '--lang=pt-BR,pt',
      '--disable-sync',
      '--disable-translate',
      '--no-first-run',
      '--safebrowsing-disable-auto-update',
      '--disable-prompt-on-repost',
      '--disable-popup-blocking',
      '--disable-default-apps',
      '--password-store=basic',
      '--use-mock-keychain',
      '--disable-crash-reporter',
      "--proxy-server='direct://'",
      '--proxy-bypass-list=*',
      '--deterministic-fetch',
    ],
    devtools: true,
    dumpio: false,
    executablePath: await chromium.executablePath,
    defaultViewport: chromium.defaultViewport,
  });

  const page = await browser.newPage();

  // Increase the default timeout limit (2 minutes)
  await page.setDefaultNavigationTimeout(120000);

  // We need to intercept the requests to store
  const cookieList = {
    strictlyNeeded: [],
    performance: [],
    functionality: [],
    publicity: [],
    socialNetwork: [],
    targeting: [],
  };

  let cookiesInside = [];

  try {
    let response = [];
    // We'll use promise race to ensure page dont exced our timeout (1.5 minute)
    response = await Promise.race([
      page.goto(link).catch((e) => void e),
      new Promise((x) => setTimeout(x, 90 * 1000)),
    ]);

    // Due to the change from waiting networkIdle to DOMContentLoadade, additional timeout
    await page.waitForTimeout(10000);

    if (response === undefined || response.status() >= 400) {
      // Let's try to open with a https - some links isnt redirecting properly
      link = link.replace('http://', 'https://');

      response = await Promise.race([
        page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
        new Promise((x) => setTimeout(x, 90 * 1000)),
      ]);

      await page.waitForTimeout(10000);

      if (response === undefined) {
        final.cookieBannerImplemented = false;
        final.siteStatus = 'offline';

        throw new Error('Unexpected problem with the link');
      }
    }

    if (response && response.status() >= 400) {
      final.cookieBannerImplemented = false;
      final.siteStatus = 'offline';

      throw new Error('Invalid HTTP Status');
    }

    const linkFinal = page.url();

    const finalUrl = new URL(linkFinal);

    const initialUrl = new URL(link);

    if (
      finalUrl.hostname.replace('www.', '') !==
      initialUrl.hostname.replace('www.', '')
    ) {
      final.cookieBannerImplemented = false;
      final.siteStatus = 'redirect';
      throw new Error('Redirect deteted');
    }

    /*
    // Go to a blank page, to ensure we'll have all the informations on the next access
    await Promise.race([
      page.goto('about:blank').catch((e) => void e),
      new Promise((x) => setTimeout(x, 5 * 1000)),
    ]);

    // We'll use promise race to ensure page dont exced our timeout (1.5 minute)
    await Promise.race([
      page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
      new Promise((x) => setTimeout(x, 90 * 1000)),
    ]);
    */

    // Due to the change from waiting networkIdle to DOMContentLoadade, additional timeout
    // await page.waitForTimeout(15000);

    /*
    const client = await page.target().createCDPSession();
    await client.send('Network.clearBrowserCookies');
    await client.send('Network.clearBrowserCache');
    */

    // Agegate bypass flow
    if (agegateData && agegateData.length > 0) {
      switch (agegateStorage) {
        case 'cookie':
        case 'cookies':
          await page.evaluate((agegateData) => {
            const d = new Date();
            d.setTime(d.getTime() + 30 * 24 * 60 * 60 * 1000);
            const expires = 'expires=' + d.toUTCString();

            agegateData.forEach((ageData) => {
              document.cookie =
                ageData.key + '=' + ageData.value + ';' + expires + ';path=/';
            });
          }, agegateData);

          break;

        case 'session_storage':
          await page.evaluate((agegateData) => {
            agegateData.forEach((ageData) => {
              sessionStorage.setItem(ageData.key, ageData.value);
            });
          }, agegateData);
          break;

        case 'local_storage':
          await page.evaluate((agegateData) => {
            agegateData.forEach((ageData) => {
              localStorage.setItem(ageData.key, ageData.value);
            });
          }, agegateData);
          break;

        case 'redirect':
          await Promise.race([
            page
              .goto(agegateData[0].value, { waitUntil: 'domcontentloaded' })
              .catch((e) => void e),
            new Promise((x) => setTimeout(x, 60 * 1000)),
          ]);
          break;

        case 'steps':
          for await (const step of agegateData) {
            switch (step.action) {
              case 'fill':
                await page.waitForSelector(step.selector, {
                  visible: true,
                  timeout: 3000,
                });
                await page.focus(step.selector);
                await page.keyboard.type(step.value, { delay: 200 });
                break;
              case 'click':
                await page.waitForSelector(step.selector);
                await page.click(step.selector);
                break;
              case 'select':
                await page.waitForSelector(step.selector, {
                  visible: true,
                  timeout: 3000,
                });
                await page.select(step.selector, step.value);
                await page.waitForTimeout(1000);
                break;
              case 'end':
                await page.waitForTimeout(5000);
                break;
              default:
                break;
            }
          }
          break;

        default:
          break;
      }
    }

    await Promise.race([
      page.goto('about:blank').catch((e) => void e),
      new Promise((x) => setTimeout(x, 5 * 1000)),
    ]);

    await Promise.race([
      page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
      new Promise((x) => setTimeout(x, 90 * 1000)),
    ]);

    /*
    await page.reload({ waitUntil: 'domcontentloaded' });
    */
    // Due to the change from waiting networkIdle to DOMContentLoadade, additional timeout
    await page.waitForTimeout(15000);

    final.cookieBannerImplemented = await page.evaluate(() => {
      // Usual case, cookie banner from Onetrust
      if (typeof Optanon !== 'undefined') {
        return true;
      }

      // Added for https://www.budweiser.jp/ (2022-11-23)
      if (
        typeof cookieconsent !== 'undefined' &&
        typeof cookieconsent.status === 'object'
      ) {
        return true;
      }

      // Added for https://abinbevefes.ru/ (2022-11-23)
      if (
        typeof Cookiebot !== 'undefined' &&
        Cookiebot.name === 'CookieConsent' &&
        typeof Cookiebot.dialog === 'object'
      ) {
        return true;
      }

      // Added for https://vezasur.com/ (2022-11-23)
      if (
        document.getElementById('cookie-notice') !== null &&
        document.getElementById('cn-notice-text') !== null &&
        document.getElementById('cn-notice-buttons') !== null
      ) {
        return true;
      }

      // Added for https://voa.ambev.com.br/ (2022-11-23)
      if (
        typeof initializeCookieModal === 'function' &&
        typeof showCookieModal === 'function' &&
        typeof hideCookieModal === 'function'
      ) {
        return true;
      }

      // Added for https://together.ambev.com.br/ (2022-11-23)
      function checkTextInButtons(text) {
        const buttons = document.getElementsByTagName('button');
        for (let index = 0; index < buttons.length; index++) {
          if (buttons[index].innerText.toLowerCase().includes(text)) {
            return true;
          }
        }
        return false;
      }

      if (
        checkTextInButtons('aceitar cookies') &&
        checkTextInButtons('configuração de cookies')
      ) {
        return true;
      }

      // Default case -- banner not implemented or not mapped case
      return false;
    });

    // TODO: drop this.
    final.cookieBannerDomain = '';
    final.cookieBannerSameDomain = false;
    // End of what we need to drop

    cookiesInside = await page.cookies();

    for await (const c of cookiesInside) {
      if (!agegateArray.includes(c.name)) {
        cookieList.strictlyNeeded.push(c.name);
      }
    }

    /*
    for await (group of groups) {
      if (final.cookieBannerImplemented && group.id > 1) {
        consentGroup = group.group;

        const iteration = await page.evaluate((consentGroup) => {
          function getCookie(cName) {
            const name = cName + '=';
            const cDecoded = decodeURIComponent(document.cookie);
            const cArr = cDecoded.split('; ');
            let res;
            cArr.forEach((val) => {
              if (val.indexOf(name) === 0) res = val.substring(name.length);
            });
            return res;
          }

          function setCookie(cName, cValue, expDays) {
            let date = new Date();
            date.setTime(date.getTime() + expDays * 24 * 60 * 60 * 1000);
            const expires = 'expires=' + date.toUTCString();
            document.cookie =
              cName +
              '=' +
              cValue +
              '; ' +
              expires +
              ';domain=.' +
              window.location.host +
              '; path=/';
          }

          if (getCookie('OptanonConsent')) {
            setCookie(
              'OptanonConsent',
              getCookie('OptanonConsent').split('groups')[0] +
                'groups=' +
                consentGroup,
              30
            );

            setCookie('OptanonAlertBoxClosed', new Date().toISOString(), 30);
          }
        }, consentGroup);

        // Agegate bypass flow
        if (agegateData && agegateData.length > 0) {
          switch (agegateStorage) {
            case 'cookie':
            case 'cookies':
              await page.evaluate((agegateData) => {
                const d = new Date();
                d.setTime(d.getTime() + 30 * 24 * 60 * 60 * 1000);
                const expires = 'expires=' + d.toUTCString();

                agegateData.forEach((ageData) => {
                  document.cookie =
                    ageData.key +
                    '=' +
                    ageData.value +
                    ';' +
                    expires +
                    ';path=/';
                });
              }, agegateData);

              break;

            case 'session_storage':
              await page.evaluate((agegateData) => {
                agegateData.forEach((ageData) => {
                  sessionStorage.setItem(ageData.key, ageData.value);
                });
              }, agegateData);
              break;

            case 'local_storage':
              await page.evaluate((agegateData) => {
                agegateData.forEach((ageData) => {
                  localStorage.setItem(ageData.key, ageData.value);
                });
              }, agegateData);
              break;

            case 'redirect':
              await Promise.race([
                page
                  .goto(agegateData[0].value, { waitUntil: 'domcontentloaded' })
                  .catch((e) => void e),
                new Promise((x) => setTimeout(x, 60 * 1000)),
              ]);
              break;
            default:
              break;
          }
        }

        await page.reload({ waitUntil: 'domcontentloaded' });

        await page.waitForTimeout(5000);

        cookiesInside = await page.cookies();

        for await (c of cookiesInside) {
          if (
            !cookieList.strictlyNeeded.includes(c.name) &&
            c.name !== 'OptanonAlertBoxClosed' &&
            !agegateArray.includes(c.name)
          )
            cookieList[group.name].push(c.name);
        }

        await client.send('Network.clearBrowserCookies');
        await client.send('Network.clearBrowserCache');

        if (group.id < 6) {
          await page.reload({ waitUntil: 'domcontentloaded' });
        }
      }
    }
    */

    final.cookies = cookieList;
    final.status = 'success';
  } catch (error) {
    if (error.message) {
      responseCode = 200;
    } else {
      responseCode = 500;
    }

    final.onetrustImplemented = false;
    final.status = 'failed';
    final.error = error.message;
  } finally {
    await browser.close();

    console.log('Finished with status: ' + responseCode);

    final = JSON.stringify(final);

    res.status(responseCode).send(final);
  }
};
