<?php

namespace App\Http\Controllers;

use App\Models\AgegateDataValidation as ModelsAgegateDataValidation;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

/**
 * AgegateData Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class AgegateDataValidation extends Controller
{
    /**
     * Store agegate data
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function store($request): JsonResponse
    {
        try {
            $ageData = new ModelsAgegateDataValidation();

            $userAgent = $request->user_agent;
            $day = $request->day;
            $month = $request->month;
            $year = $request->year;
            $domain = $request->domain;

            if (isset($userAgent) && strlen($userAgent) > 0) {
                $ageData->user_agent = $userAgent;
            } else {
                $ageData->user_agent = '-';
            }

            if (isset($day) && strlen($day) > 0) {
                $ageData->day = $day;
            } else {
                $ageData->day = '-';
            }


            if (isset($month) && strlen($month) > 0) {
                $ageData->month = $month;
            } else {
                $ageData->month = '-';
            }

            if (isset($year) && strlen($year) > 0) {
                $ageData->year = $year;
            } else {
                $ageData->year = '-';
            }

            if (isset($domain) && strlen($domain) > 0) {
                $ageData->domain = $domain;
            } else {
                $ageData->domain = '-';
            }

            $ageData->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
