<?php

// phpcs:disable PSR2.ControlStructures.ControlStructureSpacing.SpacingAfterOpenBrace

namespace App\Http\Controllers;

use App\Models\Criteria;
use Exception;
use Illuminate\Http\JsonResponse;

/**
 * Criteria Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class CriteriaController extends Controller
{
    /**
     * Get list of criterias
     *
     * @return JsonResponse
     */
    public function getCriteriasList(): JsonResponse
    {
        $return = [];
        $statusCode = 200;

        try {
            $criterias = Criteria::orderByRaw('UPPER(name) ASC')->get(['name', 'id']);

            $return = [
                'status' => 'success',
                'data' => array(
                    'criterias' => $criterias,
                ),
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
            ];
            $statusCode = 500;
        } finally {
            return response()->json($return, $statusCode);
        }
    }
}
