<?php

namespace App\Http\Controllers;

use App\Exports\ReportExport;
use App\Http\Controllers\Helpers\HelpersController;
use App\Http\Controllers\KeywordsController;
use App\Models\Links;
use App\Models\Logs;
use App\Models\Ownership;
use App\Models\Requests;
use App\Models\ssoSessions;
use DateTime;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * TicketsController Controller
 *
 * @author Tércio Lacerda <Tercio.lacerda-ext@ab-inbev.com>
 * @package App\Http\Controllers
*/
class TicketsController extends Controller
{
	/**
	 * Map of request parameters to respective table.column in db
	*/
	protected $bdMapColumns = [
		'text' => 'links.link',
		'ticketType' => 'requests.request_type',
		'ticketStatus' => 'requests.request_status',
		'dateRange' => 'requests.created_at',
		'users' => 'users.email',
	];

	/**
	 * Create a ticket for change asset details
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function createAssetDetailsTicket(Request $request): JsonResponse
	{
		try {
			$params = $request->all();
			// $sessionId = $request->header('sessionId');
			// if (!$sessionId) throw new Exception('Session ID not founded.', 1);
			// $sessionId = Crypt::decryptString(urldecode($sessionId), true);
			// $session = ssoSessions::where('session_identifier', $sessionId)->latest('id')->first();
			// if (!$session) throw new Exception('Session not founded.', 1);

			if ((empty($params['linkId']) || strlen($params['linkId']) === 0) || (empty($params['fieldsToChange']) || strlen($params['fieldsToChange']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			$details = array(
				'feedback' => null,
				'fields' => $params['fieldsToChange'],
			);

			$newReq = new Requests();
			$newReq->request_type = 'asset-details-ticket';
			$newReq->user_id = $request->userReq->userId;
			$newReq->details = json_encode($details);
			$newReq->link_id = $params['linkId'];
			$newReq->save();

			$newLog = new Logs();
			$newLog->log_type = 'open';
			$newLog->user_id = $request->userReq->userId;
			$newLog->request_id = $newReq->id;
			$newLog->save();

			$statusCode = 200;
			$return = [
					'status' => 'success',
					'message' => __('Created successfully'),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Create a ticket for change asset details
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function createDisproveResultTicket(Request $request): JsonResponse
	{
		try {
			$params = $request->all();

			if ((empty($params['link']) || strlen($params['link']) === 0) || (empty($params['reportDate']) || strlen($params['reportDate']) === 0) || (empty($params['criterias']) || strlen($params['criterias']) === 0) || (empty($params['page']) || strlen($params['page']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			$link = Links::where('link', $params['link'])->latest('id')->first();
			if (!$link) throw new Exception('Link not founded.', 1);

			$details = array(
				'feedback' => null,
				'criterias' => $params['criterias'],
				'reportDate' => $params['reportDate'],
				'page' => $params['page'],
			);

			$newReq = new Requests();
			$newReq->request_type = 'disprove-result-ticket';
			$newReq->user_id = $request->userReq->userId;
			$newReq->details = json_encode($details);
			$newReq->link_id = $link->id;
			$newReq->save();

			$newLog = new Logs();
			$newLog->log_type = 'open';
			$newLog->user_id = $request->userReq->userId;
			$newLog->request_id = $newReq->id;
			$newLog->save();

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'message' => __('Created successfully'),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Create a ticket for change asset details
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function createKeywordsTicket(Request $request): JsonResponse
	{
		try {
			$params = $request->all();

			if ((empty($params['action']) || strlen($params['action']) === 0) || (empty($params['keywordForm']) || count($params['keywordForm']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			$details = array(
				'feedback' => null,
				'action' => $params['action'],
				'keywordForm' => $params['keywordForm'],
				'differences' => $params['differences'] ? $params['differences'] : null,
			);

			$newReq = new Requests();
			$newReq->request_type = 'keywords-ticket';
			$newReq->user_id = $request->userReq->userId;
			$newReq->details = json_encode($details);
			$newReq->link_id = null;
			$newReq->save();

			$newLog = new Logs();
			$newLog->log_type = 'open';
			$newLog->user_id = $request->userReq->userId;
			$newLog->request_id = $newReq->id;
			$newLog->save();

			$statusCode = 200;
			$return = [
					'status' => 'success',
					'message' => __('Created successfully'),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Create a ticket for change asset details
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function createOwnershipTicket(Request $request): JsonResponse
	{
		try {
			$params = $request->all();

			if ((empty($params['linkId']) || strlen($params['linkId']) === 0) || (empty($params['action']) || strlen($params['action']) === 0) || (empty($params['criterias']) || count($params['criterias']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			if ($params['action'] === 'update') {
				if ((empty($params['name']) || strlen($params['name']) === 0) || (empty($params['email']) || strlen($params['email']) === 0)) throw new Exception('Requireds params are missing.', 1);
			}

			$details = array(
				'feedback' => null,
				'action' => $params['action'],
				'name' => !empty($params['name']) ? $params['name'] : '',
				'email' => !empty($params['email']) ? $params['email'] : '',
				'criterias' => $params['criterias'],
				'linkId' => $params['linkId'],
			);

			$newReq = new Requests();
			$newReq->request_type = 'ownership-ticket';
			$newReq->user_id = $request->userReq->userId;
			$newReq->details = json_encode($details, JSON_UNESCAPED_UNICODE);
			$newReq->link_id = $params['linkId'];
			$newReq->save();

			$newLog = new Logs();
			$newLog->log_type = 'open';
			$newLog->user_id = $request->userReq->userId;
			$newLog->request_id = $newReq->id;
			$newLog->save();

			$statusCode = 200;
			$return = [
					'status' => 'success',
					'message' => __('Created successfully'),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Export tickets asset details
	 *
	 * @param Request $request
	 *
	 * @return BinaryFileResponse|JsonResponse
	*/
	public function exportAssetDetailsTickets(Request $request): BinaryFileResponse|JsonResponse
	{
		try {
			$tickets = Requests::query()
				->leftJoin('links', 'requests.link_id', '=', 'links.id')
				->join('users', 'requests.user_id', '=', 'users.id');

			HelpersController::buildRequestWhere($tickets, $request->all(), $this->bdMapColumns, 'export-ticket-asset-details');

			$tickets = $tickets->orderBy('requests.created_at')
				->get([
					'links.link as asset',
					'requests.details as reqDetails',
					'requests.created_at as reqCreatedAt',
					'users.email as reqAuthor',
				])
				->toArray();

			$toExport = array();
			$headers = array(
				'Asset',
				'Country',
				'Brand',
				'Department',
				'Environment',
				'Status',
				'DGP Scope',
				'Alcoholic',
				'Requested at',
				'Requested by',
			);
			array_push($toExport, $headers);

			foreach ($tickets as $key => $value) {
				$asset = $value['asset'];
				$reqCreatedAt = $value['reqCreatedAt'];
				$reqAuthor = $value['reqAuthor'];
				$fields = json_decode(json_decode($value['reqDetails'], true)['fields'], true);
				$arrAsset = array();
				for ($i = 0; $i < count($headers); $i++) {
					array_push($arrAsset, null);
				}
				$arrAsset[0] = $asset;
				$arrAsset[8] = $reqCreatedAt;
				$arrAsset[9] = $reqAuthor;
				foreach ($fields as $keyField => $field) {
					$field = $field['update'];
					$valueToUpdate = '';
					if (!empty($field['text'])) $valueToUpdate = $field['text'];
					if (empty($field['text'])) $valueToUpdate = $field;

					if ($keyField === 'alcoholic') $arrAsset[7] = $valueToUpdate === true ? 'Alcoholic' : 'Not alcoholic';
					if ($keyField === 'brand') $arrAsset[2] = $valueToUpdate;
					if ($keyField === 'country') $arrAsset[1] = $valueToUpdate;
					if ($keyField === 'department') $arrAsset[3] = $valueToUpdate;
					if ($keyField === 'dgp_goal') $arrAsset[6] = $valueToUpdate === true ? 'Considered' : 'Not considered';
					if ($keyField === 'environment') $arrAsset[4] = $valueToUpdate;
					if ($keyField === 'status') $arrAsset[5] = $valueToUpdate;
				}
				array_push($toExport, $arrAsset);
			}

			$report = new ReportExport($toExport);
			$date = new DateTime();
			$filename = 'export-ticket-asset-details' . '-' . $date->format('Y-m-d') . '.xlsx';
			$return = Excel::download($report, $filename);
		} catch (Exception $e) {
			$return = response()->json([
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			]);
		}
		return $return;
	}

	/**
 		* Return all details of a specific ticket (Asset Details type)
  	*
		* @param int $id - Ticket Id
	 	*
	 	* @return JsonResponse
	*/
	public function getAssetDetailsTicketInfo(int $id): JsonResponse
	{
		try {
			$ticket = Requests::join('links', 'links.id', '=', 'requests.link_id')
				->join('brands', 'brands.id', '=', 'links.brand_id')
				->join('channels', 'channels.id', '=', 'links.channel_id')
				->join('countries', 'countries.id', '=', 'links.country_id')
				->join('zones', 'zones.id', '=', 'countries.zone_id')
				->where('requests.id', $id)
				->first([
					'requests.id as reqId',
					'requests.created_at as reqCreatedAt',
					'requests.request_status as reqStatus',
					'requests.updated_at as reqLastUpdate',
					'links.link as asset',
					'requests.request_type as type',
					'zones.name as zone',
					'countries.name as country',
					'channels.name as channel',
					'brands.name as brand',
					'links.http_status as httpStatus',
					'links.consider as dgpGoal',
					'links.environment as environment',
					'links.scope as department',
					'links.alcoholic as alcoholic',
				]);
				$statusCode = 200;
				$return = [
					'status' => 'success',
					'data' => array(
						'ticket' => $ticket
					),
				];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Return all tickets paginateds (Only to Admin and SuperAdmin user's)
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function getTickets(Request $request): JsonResponse
	{
		try {
			$tickets = Requests::query()
				->leftJoin('links', 'requests.link_id', '=', 'links.id')
				->join('users', 'requests.user_id', '=', 'users.id');

			HelpersController::buildRequestWhere($tickets, $request->all(), $this->bdMapColumns);

			$tickets = $tickets->orderByDesc('requests.created_at')
				->paginate($request->per_page, [
					'requests.id as reqId',
					'requests.details as reqDetails',
					'requests.created_at as reqCreatedAt',
					'requests.request_status as reqStatus',
					'requests.updated_at as reqLastUpdate',
					'links.link as asset',
					'requests.request_type as type',
					'users.name as reqUser',
					'users.email as reqUserEmail',
				]);

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'data' => array(
					'tickets' => $tickets
				),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Return a paginated tickets by a user
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function getUserTickets(Request $request): JsonResponse
	{
		try {
			$tickets = Requests::query()
				->leftJoin('links', 'requests.link_id', '=', 'links.id')
				->where(function ($query) use ($request) {
					$query->where('requests.user_id', '=', $request->userReq->userId);
				});

			HelpersController::buildRequestWhere($tickets, $request->all(), $this->bdMapColumns);

			$tickets = $tickets->orderByDesc('requests.created_at')
				->paginate($request->per_page, [
					'requests.id as reqId',
					'requests.details as reqDetails',
					'requests.created_at as reqCreatedAt',
					'requests.request_status as reqStatus',
					'requests.updated_at as reqLastUpdate',
					'links.link as asset',
					'requests.request_type as type',
				]);

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'data' => array(
					'tickets' => $tickets
				),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Update a ticket with a Admin/SuperAdmin response
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function updateKeywordsTicketResult(Request $request): JsonResponse
	{
		try {
			$params = $request->all();
			if ((empty($params['review']) || strlen($params['review']) === 0) || (empty($params['reqId']) || strlen($params['reqId']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			$reqId = $params['reqId'];
			$review = $params['review'];
			$feedbackMessage = $params['feedbackMessage'];

			$newLog = new Logs();
			$newLog->user_id = $request->userReq->userId;
			$newLog->request_id = $reqId;
			$newLog->log_type = $review;
			$newLog->save();

			$ticketReviewed = tap(Requests::where('id', $reqId))->update(['approved' => $review === 'approved' ? true : false, 'request_status' => $review])->first();
			$details = json_decode($ticketReviewed->details, true);
			if ($review === 'rejected') {
				$details['feedback'] = $feedbackMessage;
				Requests::where('id', $reqId)->update(['details' => json_encode($details)]);
			} else {
				$details['feedback'] = null;
				Requests::where('id', $reqId)->update(['details' => json_encode($details)]);
			}

			if ($review === 'approved') {
				$action = $details['action'];
				$keywordForm = $details['keywordForm'];
				$kwController = new KeywordsController();
				if ($action === 'create') {
					$kwController->store(json_encode($keywordForm), true);
				} elseif ($action === 'update') {
					$kwController->update($keywordForm['id'], json_encode($keywordForm), true);
				} elseif ($action === 'delete') {
					$kwController->destroy($keywordForm['id']);
				}
			}

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
				'ticket' => $ticketReviewed,
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Update a ownership ticket with a Admin/SuperAdmin response
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function updateTicketOwnership(Request $request): JsonResponse
	{
		try {
			$params = $request->all();
			if ((empty($params['review']) || strlen($params['review']) === 0) || (empty($params['reqId']) || strlen($params['reqId']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			$reqId = $params['reqId'];
			$review = $params['review'];
			$feedbackMessage = $params['feedbackMessage'];

			$newLog = new Logs();
			$newLog->user_id = $request->userReq->userId;
			$newLog->request_id = $reqId;
			$newLog->log_type = $review;
			$newLog->save();

			$ticketReviewed = tap(Requests::where('id', $reqId))->update(['approved' => $review === 'approved' ? true : false, 'request_status' => $review])->first();
			$details = json_decode($ticketReviewed->details, true);
			if ($review === 'rejected') {
				$details['feedback'] = $feedbackMessage;
				Requests::where('id', $reqId)->update(['details' => json_encode($details, JSON_UNESCAPED_UNICODE)]);
			} else {
				$details['feedback'] = null;
				Requests::where('id', $reqId)->update(['details' => json_encode($details, JSON_UNESCAPED_UNICODE)]);
				$owner = [
					'name' => $details['name'],
					'email' => $details['email'],
				];
				$update = array();
				foreach ($details['criterias'] as $key => $value) {
					if (empty($update[$value['value']])) $update[$value['value']] = $details['action'] === 'update' ? json_encode($owner, JSON_UNESCAPED_UNICODE) : NULL;
				}
				Ownership::where('link_id', $details['linkId'])->update($update);
			}

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
				'ticket' => $ticketReviewed,
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Update a ticket with a Admin/SuperAdmin response
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	*/
	public function updateTicketResult(Request $request): JsonResponse
	{
		try {
			$params = $request->all();
			if ((empty($params['review']) || strlen($params['review']) === 0) || (empty($params['reqId']) || strlen($params['reqId']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			$reqId = $params['reqId'];
			$review = $params['review'];
			$feedbackMessage = $params['feedbackMessage'];

			$newLog = new Logs();
			$newLog->user_id = $request->userReq->userId;
			$newLog->request_id = $reqId;
			$newLog->log_type = $review;
			$newLog->save();

			$ticketReviewed = tap(Requests::where('id', $reqId))->update(['approved' => $review === 'approved' ? true : false, 'request_status' => $review])->first();
			$details = json_decode($ticketReviewed->details, true);
			if ($review === 'rejected') {
				$details['feedback'] = $feedbackMessage;
				Requests::where('id', $reqId)->update(['details' => json_encode($details)]);
			} else {
				$details['feedback'] = null;
				Requests::where('id', $reqId)->update(['details' => json_encode($details)]);
			}

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
				'ticket' => $ticketReviewed,
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}
}
