<?php

namespace App\Http\Middleware;

use App\Http\Controllers\ApiTokenController;
use App\Models\ApiRequest;
use Closure;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;

class UsersTokenValidate
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure(\Illuminate\Http\Request): (\Illuminate\Http\Response|\Illuminate\Http\RedirectResponse)  $next
     * @return RedirectResponse|JsonResponse
     */
    public function handle(Request $request, Closure $next)
    {
        try {

            if (is_null($request->bearerToken())) {
                throw new Exception('Failed to retrieve Bearer token');
            }

            if (empty($request->bearerToken())) {
                throw new Exception('Please check the Bearer token');
            }

            $validatedUser = app(ApiTokenController::class)->validateBearer($request->bearerToken());

            if (is_null($validatedUser)) {
                throw new Exception('Failed to validate the Bearer token');
            }

            $url = $request->getPathInfo();

            $this->logRequest($validatedUser->token_id, $url);

            return $next($request);
        } catch (Exception $e) {
            return response()->json([
                'status' => 'error',
                'message' => $e->getMessage(),
            ], 401);
        }
    }

    protected function logRequest(int $tokenId, string $route)
    {
        $created = false;

        try {
            $entry = new ApiRequest();

            $entry->token_id = $tokenId;
            $entry->route = $route;

            $entry->save();

            $created = true;
        } catch (Exception $e) {
            $created = false;
        } finally {
            return $created;
        }
    }
}
