<?php

namespace App\Http\Middleware;

use App\Http\Controllers\Auth\UserAuthController;
use App\Models\ssoSessions;
use App\Models\User;
use Auth;
use Closure;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;

class ValidateApiLogin
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle(Request $request, Closure $next)
    {
        $return = [];
        $statusCode = 0;

        try {
            $data = app(UserAuthController::class)->sso_validateToken($request, false, true)->getContent();
            $data = json_decode($data);
            $user = null;
            if (!$data && !$data->user) { // @phpstan-ignore-line
                $sessionIdentifier = explode('|', Crypt::decryptString(urldecode($request->header('ssoToken'))));
                $sessionIdentifier = $sessionIdentifier[count($sessionIdentifier) - 1];

                $session = Crypt::decryptString(urldecode($data->sessionId)); // @phpstan-ignore-line
                $session = ssoSessions::where('session_identifier', $session)->latest('id')->first();
                $user = User::join('user_level', 'users.user_level_id', '=', 'user_level.id')
                    ->where('users.id', $session->user_id)
                    ->first([
                        'users.id as userId',
                        'user_level.level as userLevel',
                        'users.name as userName',
                        'users.email as userEmail',
                    ]);
            } else {
                $user = $data->user;
            }

            $request->userReq = $user;

            if ($data->status === 'success') {
                $statusCode = 200;
            } else {
                $return = [
                    'status' => 'error',
                    'message' => 'invalid token'
                ];

                $statusCode = 401;
            }
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => $e->getMessage(),
                'line' => $e->getLine(),
            ];

            $statusCode = 500;
        }

        if ($statusCode !== 200) {
            return response()->json($return, $statusCode);
        } else {
            return $next($request);
        }
    }
}
