# Google Lighthouse

Our current implementation of Google Lighthouse takes leverage of the parallel execution of GCP CF, which we have described [here](https://dgp.ab-inbev.com/php/docs/getting-started/system-requirements.html#cloud-function-configuration).

We've created a specific table (**lighthouse_data**) to store the output of the Google Lighthouse in the GCP CF, the structure of the table is below:

<img :src="$withBase('/images/lighthouse-data-structure.png#center')" alt="Lighthouse-Data Table Structure">

This way, we can have more executions from Google Lighthouse than the other robots - one thing that our stakeholders have requested from us, they wanted to have more visibility from their efforts to fix the performance issues from the website.

To achieve that, we've in the Dockerfile of PHP Container one file that creates crontab entries to run 3 times per week on the production environment:

```dockerfile
RUN if [ "$env_stage" = "prod" ]; then (crontab -l 2>/dev/null; echo "0 6 * * 2 cd /var/www/html; /usr/local/bin/node ./app/Console/Js/parallel-cf-lighthouse.js >> /var/log/cron.log 2>&1") | crontab -; fi
RUN if [ "$env_stage" = "prod" ]; then (crontab -l 2>/dev/null; echo "0 6 * * 4 cd /var/www/html; /usr/local/bin/node ./app/Console/Js/parallel-cf-lighthouse.js >> /var/log/cron.log 2>&1") | crontab -; fi
RUN if [ "$env_stage" = "prod" ]; then (crontab -l 2>/dev/null; echo "0 6 * * 6 cd /var/www/html; /usr/local/bin/node ./app/Console/Js/parallel-cf-lighthouse.js >> /var/log/cron.log 2>&1") | crontab -; fi
```

For this script to work as expected, is necessary to ensure that the .env file has been configured correctly with all the needed entries. In the AbInbev scenario, we've deployed the Google Lighthouse to the GCP CF with this [hardware specification](https://dgp.ab-inbev.com/php/docs/docs/getting-started/system-requirements.html#cloud-function-configuration) and in three different regions:

- North America
- Europe
- Asia

We need to have a different server, in different locations, due to the score variability of the Google Lighthouse based on the latency and geographical distance between the client and the server - [reference](https://github.com/GoogleChrome/lighthouse/blob/main/docs/variability.md).

In case that is necessary to add new regions, all that needs to be done is add the **3 entries** in the **.env** and update the **parallel-cf-lighthouse.js**, basically in the **resolveRegion** function:

```javascript
async function resolveRegion(zone, inAcquia) {
  zone = zone.toLowerCase();

  const cloudLocation = {
    region: '',
    script: '',
    url: '',
  };

  if (inAcquia || zone === 'Europe') {
    cloudLocation.region = process.env.GCP_CF_LIGHTHOUSE_EUR_REGION;
    cloudLocation.script = process.env.GCP_CF_LIGHTHOUSE_EUR_SCRIPT;
    cloudLocation.url = process.env.GCP_CF_LIGHTHOUSE_EUR_SCRIPTv2;
  } else if (zone === 'Asia Pacific') {
    cloudLocation.region = process.env.GCP_CF_LIGHTHOUSE_APAC_REGION;
    cloudLocation.script = process.env.GCP_CF_LIGHTHOUSE_APAC_SCRIPT;
    cloudLocation.url = process.env.GCP_CF_LIGHTHOUSE_APAC_SCRIPTv2;
  } else {
    cloudLocation.region = process.env.GCP_CF_LIGHTHOUSE_NAZ_REGION;
    cloudLocation.script = process.env.GCP_CF_LIGHTHOUSE_NAZ_SCRIPT;
    cloudLocation.url = process.env.GCP_CF_LIGHTHOUSE_NAZ_SCRIPTv2;
  }

  return cloudLocation;
}
```
