# System Requirements

This project has been designed to be multi-cloud.

Specifically in the AbInbev, we're using an [Azure](https://azure.microsoft.com/en-us) and [Google Cloud Platform](https://cloud.google.com/):

- Azure is the main hosting, where is hosted the application and all our databases.
- GCP is running some independent tasks (Website performance and cookie audit).

## Application Requirements

We've optimized the application and reduced the system requirements, to ensure that we're running everything in a smooth experience for our users and with minimal costs.

In the current state, the major requirements are due to the need to run a Google Chromium instance to audit our assets.

We're running without any major issues, our current application, in one server with the following specification:

- CPU: 2 vCPUs
- RAM: 4 GiB memory

That's enough to run our application (backend + frontend) and audit one asset, without problems for the users.

## Google Cloud Platform

After comparing the major cloud providers available, we've decided to use GCP because in our case the performance of GCP was better than the alternatives.

We're using one service from GCP: [Cloud Functions](https://cloud.google.com/functions), the reasons for that:

- Site loading performance is sensitive to the distance between the server and client, GCP CF allows us to deploy our code in multiple places around the world - [reference](https://github.com/GoogleChrome/lighthouse/blob/main/docs/variability.md);
  GCP CF allows us to run audit multiple assets in parallel, reducing the total time to audit all our properties;
- GCP CF is paid only by usage.

---

The code deployed to GCP CF is located at:

- Google Pagespeed (Lighthouse): app/Console/Js/cf-lighthouse.js
- Cookie audit: app/Console/Js/cf-cookiecrawler.js

### Cloud Function Configuration

Our code has been deployed on a [2nd Generation Cloud Function](https://cloud.google.com/blog/products/serverless/cloud-functions-2nd-generation-now-generally-available), due to:

- Better performance when compared with the 1st generation;
- More hardware capability available;
- A bigger number of simultaneous instances.

The hardware specification used:

- CPU: 4 CPU\*
- RAM: 16GiB memory\*
- Maximum concurrent request per instance: 1\*\*
- Minimum number of instances: 0\*\*\*
- Maximum number of instances: 100\*\*\*\*

::: details

\* With this configuration, we've enough hardware power to retrieve the site performance

\*\* With this configuration, we'll run only one instance of our audit per server

\*\*\* With this configuration, we'll not keep a server running - only paying when being used

\*\*\*\* This configuration allows us to run 100 different audits at the same time
:::
