/* eslint-disable import/no-absolute-path */
import { createRouter, createWebHistory } from 'vue-router';
import store from '../store';
import AuthLayout from '/src/layout/auth-layout.vue';
import AppLayout from '/src/layout/app-layout.vue';

function generateRoutes() {
  const entries = [
    {
      name: 'Admin',
      path: '/admin',
      // component: () => import('/src/pages/keywords.vue'),
      meta: {
        icon: 'admin_panel_settings',
        appearInMenu: true,
        collapse: true,
        collapseParent: true,
        collapseType: 'admin',
        routeType: 'Admin',
      },
    },
    {
      name: 'NPS',
      path: '/admin/nps',
      component: () => import('/src/pages/admin/nps.vue'),
      meta: {
        icon: 'chat',
        appearInMenu: true,
        collapse: true,
        collapseParent: false,
        collapseType: 'admin',
        routeType: 'SuperAdmin',
      },
    },
    {
      name: 'Import BP',
      path: '/admin/import-brand-protection',
      component: () => import('/src/pages/admin/import-bp.vue'),
      meta: {
        icon: 'upload',
        appearInMenu: true,
        collapse: true,
        collapseParent: false,
        collapseType: 'admin',
        routeType: 'SuperAdmin',
      },
    },
    {
      name: 'Manual Validation',
      path: '/admin/manual-validation',
      component: () => import('/src/pages/admin/manual-validation.vue'),
      meta: {
        icon: 'done',
        appearInMenu: true,
        collapse: true,
        collapseParent: false,
        collapseType: 'admin',
        routeType: 'SuperAdmin',
      },
    },
    {
      name: 'Tickets',
      path: '/admin/tickets',
      component: () => import('/src/pages/admin/tickets.vue'),
      meta: {
        icon: 'confirmation_number',
        appearInMenu: true,
        collapse: true,
        collapseParent: false,
        collapseType: 'admin',
        routeType: 'Admin',
      },
    },
    {
      name: 'Users',
      path: '/admin/users',
      component: () => import('/src/pages/admin/users.vue'),
      meta: {
        icon: 'group',
        appearInMenu: true,
        collapse: true,
        collapseParent: false,
        collapseType: 'admin',
        routeType: 'SuperAdmin',
      },
    },
    {
      path: '/',
      name: 'Home',
      component: () => import('/src/pages/home.vue'),
      meta: {
        icon: 'home',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    {
      name: 'Results by Web Property',
      path: '/results-by-web-property',
      component: () => import('/src/pages/results-by-web-property.vue'),
      meta: {
        icon: 'line_axis',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    {
      name: 'Results by Country',
      path: '/results-by-country',
      component: () => import('/src/pages/results-by-country.vue'),
      meta: {
        icon: 'line_axis',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    {
      name: 'Web Properties List',
      path: '/web-properties-list',
      component: () => import('/src/pages/web-properties-list.vue'),
      meta: {
        icon: 'list',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    {
      name: 'Keywords',
      path: '/keywords',
      component: () => import('/src/pages/keywords.vue'),
      meta: {
        icon: 'translate',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    {
      name: 'lighthouseReport',
      path: '/lighthouse/report/:link/:executed_at',
      component: () => import('/src/pages/lighthouse/report.vue'),
      meta: {
        icon: '',
        appearInMenu: false,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    {
      name: 'Reports',
      path: '',
      // component: () => import('/src/pages/keywords.vue'),
      meta: {
        icon: 'dashboard',
        appearInMenu: true,
        collapse: true,
        collapseParent: true,
        collapseType: 'reports',
        routeType: 'Default',
      },
    },
    {
      name: 'API Token',
      path: '/api-token',
      component: () => import('/src/pages/api-token.vue'),
      meta: {
        icon: 'webhook',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    {
      name: 'My Tickets',
      path: '/my-tickets',
      component: () => import('/src/pages/my-tickets.vue'),
      meta: {
        icon: 'confirmation_number',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
    // {
    //   name: 'Locales',
    //   path: '/locales',
    //   component: () => import('/src/pages/locales/index.vue'),
    //   meta: {
    //     icon: 'travel_explore',
    //     appearInMenu: false,
    //     collapse: false,
    //     collapseParent: false,
    //     collapseType: '',
    //     routeType: 'Default',
    //   },
    // },
    // {
    //   name: 'Brands',
    //   path: '/brands',
    //   component: () => import('/src/pages/brands/index.vue'),
    //   meta: {
    //     icon: 'branding_watermark',
    //     appearInMenu: false,
    //     collapse: false,
    //     collapseParent: false,
    //     collapseType: '',
    //     routeType: 'Default',
    //   },
    // },
    {
      name: 'Logout',
      path: '/logout',
      meta: {
        icon: 'logout',
        appearInMenu: true,
        collapse: false,
        collapseParent: false,
        collapseType: '',
        routeType: 'Default',
      },
    },
  ];

  if (import.meta.env.VITE_IARD_MODULE === '1') {
    entries.push({
      name: 'IARD',
      path: '/reports/iard',
      component: () => import('/src/pages/reports/iard.vue'),
      meta: {
        icon: 'arrow_forward',
        appearInMenu: true,
        collapse: true,
        collapseParent: false,
        collapseType: 'reports',
        routeType: 'Default',
      },
    });
  }

  if (import.meta.env.VITE_EXCELLENCE_MODULE === '1') {
    entries.push(
      {
        name: 'Page Speed',
        path: '/reports/page-speed',
        component: () => import('/src/pages/reports/page-speed.vue'),
        meta: {
          icon: 'arrow_forward',
          appearInMenu: true,
          collapse: true,
          collapseParent: false,
          collapseType: 'reports',
          routeType: 'Default',
        },
      },
      {
        name: 'Excellence',
        path: '/reports/excelence',
        component: () => import('/src/pages/reports/excelence.vue'),
        meta: {
          icon: 'arrow_forward',
          appearInMenu: true,
          collapse: true,
          collapseParent: false,
          collapseType: 'reports',
          routeType: 'Default',
        },
      }
    );
  }

  if (import.meta.env.VITE_PRIVACY_MODULE === '1') {
    entries.push(
      {
        name: 'Privacy and Security',
        path: '/reports/privacy-and-security',
        component: () => import('/src/pages/reports/privacy-and-security.vue'),
        meta: {
          icon: 'arrow_forward',
          appearInMenu: true,
          collapse: true,
          collapseParent: false,
          collapseType: 'reports',
          routeType: 'Default',
        },
      },
      {
        name: 'Cookie Compliance',
        path: '/reports/cookie-compliance',
        component: () => import('/src/pages/reports/cookie-compliance.vue'),
        meta: {
          icon: 'arrow_forward',
          appearInMenu: true,
          collapse: true,
          collapseParent: false,
          collapseType: 'reports',
          routeType: 'Default',
        },
      }
    );
  }

  if (import.meta.env.VITE_DATA_MODULE === '1') {
    entries.push(
      {
        name: 'Website Data',
        path: '/reports/website-data',
        component: () => import('/src/pages/reports/website-data.vue'),
        meta: {
          icon: 'arrow_forward',
          appearInMenu: true,
          collapse: true,
          collapseParent: false,
          collapseType: 'reports',
          routeType: 'Default',
        },
      },
      {
        name: 'Google Analytics',
        path: '/reports/google-analytics',
        component: () => import('/src/pages/reports/google-analytics.vue'),
        meta: {
          icon: 'arrow_forward',
          appearInMenu: true,
          collapse: true,
          collapseParent: false,
          collapseType: 'reports',
          routeType: 'Default',
        },
      }
    );
  }

  return entries;
}

const routes = [
  {
    path: '/auth',
    name: 'auth',
    component: AuthLayout,
    children: [
      {
        name: 'Login',
        path: 'login',
        component: () => import('/src/pages/Auth/Login.vue'),
      },
      {
        path: '',
        redirect: { name: 'Login' },
      },
    ],
  },
  {
    name: 'App',
    path: '/',
    component: AppLayout,
    children: generateRoutes(),
  },
];

const router = createRouter({
  history: createWebHistory(),
  routes,
});

router.beforeEach((to, from, next) => {
  const apiUrl = import.meta.env.VITE_APIURL;

  if (to.path === '/logout') {
    logout('/');
    return;
  }

  if (to.path === '/auth/login') {
    return next();
  }

  let ssoToken = localStorage.getItem('session_identifier') || '';

  if (!ssoToken) {
    const match = document.cookie.match(/(^| )session_identifier=([^;]+)/);
    if (match) ssoToken = match[2];
  }

  let ssoExpires = localStorage.getItem('sso_expires') || '';
  if (!ssoExpires) {
    const match = document.cookie.match(/(^| )sso_expires=([^;]+)/);
    if (match) ssoExpires = match[2];
  }

  if (ssoToken && ssoToken.length > 0) {
    fetch(apiUrl + '/auth/validate-sso-token', {
      method: 'POST',
      headers: {
        ssoExpires,
        ssoToken,
      },
    })
      .then((response) => response.json())
      .then((json) => {
        if (json.status === 'success') {
          const { userId, userLevel, sessionId } = json;
          if (json.action === 'renewed') {
            localStorage.setItem('session_identifier', sessionId);
            localStorage.setItem('sso_expires', json.expires);
          }

          window.dataLayer = window.dataLayer || [];
          const checkIfLoggedInBefore = window.dataLayer.filter(
            (item) =>
              item.event_category &&
              item.event_category === 'Login' &&
              item.event_action &&
              item.event_action === 'Success'
          );

          if (checkIfLoggedInBefore.length === 0) {
            window.dataLayer.push({
              event: 'GAEvent',
              event_category: 'Login',
              event_action: 'Success',
              event_label: 'Internal SSO',
              userUid: userId,
            });
          }

          localStorage.setItem('user_id', userId);
          store.commit('updateUserId', userId);
          store.commit('updateUserLevel', userLevel);
          store.commit('updateSessionId', sessionId);

          if (userLevel === 'Admin' && to.meta.routeType === 'SuperAdmin') {
            next({ name: 'Home' });
            return;
          } else if (
            userLevel === 'Default' &&
            (to.meta.routeType === 'Admin' ||
              to.meta.routeType === 'SuperAdmin')
          ) {
            next({ name: 'Home' });
            return;
          }

          next();
        } else {
          logout(to.path);
        }
      });
  } else {
    store.commit('updateUserLevel', '');
    store.commit('updateUserId', null);
    router.push({ path: '/auth/login', query: { to: to.path } });
  }
});

function logout(to) {
  localStorage.removeItem('session_identifier');
  localStorage.removeItem('sso_expires');
  localStorage.removeItem('user_id');

  document.cookie = 'session_identifier=; Max-Age=-99999999;';
  document.cookie = 'sso_expires=; Max-Age=-99999999;';

  store.commit('updateUserLevel', '');
  store.commit('updateUserId', null);
  store.commit('updateSessionId', null);

  sessionStorage.removeItem('navCount');
  localStorage.removeItem('lastNpsFeedback');

  router.push({ path: '/auth/login', query: { to } });
}

export default router;
